/*
 * Name:    mt_timer.c
 *
 * Purpose: Tcl interface to generic MX timer functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 1999-2002 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>

#include <tcl.h>

#include "mx_util.h"
#include "mx_record.h"
#include "mx_timer.h"
#include "mx_handle.h"
#include "mx_tcl.h"
#include "mx_tcl_private.h"

MX_EXPORT int
Mx_timer_is_busy_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	int busy;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: timer_is_busy record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_timer_is_busy( record, &busy );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetIntObj( resultPtr, busy );

	return TCL_OK;
}

MX_EXPORT int
Mx_timer_start_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	const char fname[] = "Mx_timer_start_cmd()";

	MX_RECORD *record;
	int tcl_status;
	double seconds;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 2, objv,
		"Usage: timer_start record_handle seconds" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetDoubleFromObj( interp, objv[2], &seconds );

	if ( tcl_status != TCL_OK ) 
		return mxtcl_object_parse_error(
					interp, objv[2], "double", fname );

	mx_status = mx_timer_start( record, seconds );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_timer_stop_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	double seconds;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: timer_stop record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_timer_stop( record, &seconds );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetDoubleObj( resultPtr, seconds );

	return TCL_OK;
}

